/* Kernel includes. */
#include "FreeRTOS.h" /* Must come first. */
#include "queue.h"    /* RTOS queue related API prototypes. */
#include "semphr.h"   /* Semaphore related API prototypes. */
#include "task.h"     /* RTOS task related API prototypes. */
#include "timers.h"   /* Software timer related API prototypes. */

#include <inttypes.h>
#include <stdint.h>
#include <stdio.h>

#include "bsl.h"
#include "unity.h" /* unit testing support functions */

/* Priorities at which the tasks are created.  The max priority can be specified
as ( configMAX_PRIORITIES - 1 ). */
#define mainTASK_A_PRIORITY (tskIDLE_PRIORITY + 1)
#define mainTASK_B_PRIORITY (tskIDLE_PRIORITY + 2)

static void prvTaskA(void *pvParameters);
static void prvTaskB(void *pvParameters);

#if configNUMBER_OF_CORES != 2
#error Require two cores be configured for FreeRTOS
#endif

TaskHandle_t taskB;
QueueHandle_t Button_Queue;

void setup_test_v795215469_001(void) {
  xTaskCreate(prvTaskA, "TaskA", configMINIMAL_STACK_SIZE, NULL,
              mainTASK_A_PRIORITY, NULL);

  xTaskCreate(prvTaskB, "TaskB", configMINIMAL_STACK_SIZE, NULL,
              mainTASK_B_PRIORITY, NULL);
  vTaskCoreAffinitySet(taskB, 0x2);
}

void setUp(void) {} /* Is run before every test, put unit init calls here. */
void tearDown(void) {
} /* Is run after every test, put unit clean-up calls here. */

int main(void) {
  initTestEnvironment();

  Button_Queue = xQueueCreate( 10, sizeof( unsigned char) );
  if (Button_Queue == NULL)
  {
      sendReport("xQueueCreate Failed\n\0", 0);
  }

  setup_test_v795215469_001();

  vTaskStartScheduler();

  /* should never reach here */
  panic_unsupported();

  return 0; // UNITY_END is unreachable via this path. a state machine and
            // counter is used so that just one child task will call it
            // instead.
}

static uint32_t taskBState = 0;
static bool isrAssertionComplete = false;

static void softwareInterruptHandlerSimple(void) {
  int i;
  UBaseType_t uxSavedInterruptStatus;
  unsigned char message;

  char strbuf_a[] = "TRACE: ISR enter\n\0";
  size_t strbuf_a_len = sizeof(strbuf_a) / sizeof(char);

  char strbuf_b[] = "TRCE: ISR exit\n\0";
  size_t strbuf_b_len = sizeof(strbuf_b) / sizeof(char);

  sendReport(strbuf_a, strbuf_a_len);
  //uxSavedInterruptStatus = taskENTER_CRITICAL_FROM_ISR();

  message = 5;
  if (xQueueSendFromISR (Button_Queue, &message, NULL) != pdTRUE) {
    sendReport("Queue Message From ISR Failed.\n\0", 0);
  }

  //taskEXIT_CRITICAL_FROM_ISR(uxSavedInterruptStatus);
  sendReport(strbuf_b, strbuf_b_len);

  isrAssertionComplete = true;
}

static void prvTaskA(void *pvParameters) {
  int handlerNum = -1;

  handlerNum = registerSoftwareInterruptHandler(softwareInterruptHandlerSimple);
  triggerSoftwareInterrupt(handlerNum);

  // idle the task
  for (;;) {
    vTaskDelay(pdMS_TO_TICKS(10));
  }
}

static void prvTaskB(void *pvParameters) {
  UBaseType_t num;
  unsigned char message;

  char str[64];

  for (;;)
  {
    num = uxQueueMessagesWaiting(Button_Queue);
    if (num > 0)
    {
      if (xQueueReceive(Button_Queue, &message, 0) == pdTRUE)
      {
        sprintf(str, "num=%d, message=%d]\n\0", (int)num, (int)message);
        sendReport(str, 0);
      }
    }
    vTaskDelay(pdMS_TO_TICKS(1000));
  }
}
